from django.utils import timezone

from pymess.backend.emails import EmailBackend
from pymess.models import EmailMessage


class DummyEmailBackend(EmailBackend):
    """
    Dummy e-mail backend used for testing environments. Backend only logs messages to the database.
    """

    def publish_message(self, message):
        self.update_message(message, state=EmailMessage.STATE.DEBUG)

    def pull_message_info(self, message):
        message.change_and_save(
            extra_sender_data={**message.extra_sender_data, 'info': {'debug': True}},
            info_changed_at=timezone.now(),
            update_only_changed_fields=True,
        )



import boto3

from django.conf import settings
from django.utils import timezone
from django.utils.encoding import force_text

from pymess.config import settings
from pymess.backend.sms import SMSBackend
from pymess.models import OutputSMSMessage


class SNSSMSBackend(SMSBackend):
    """
    SMS backend implementing AWS SNS service via boto3 library https://aws.amazon.com/sns/
    """

    sns_client = None

    def _get_sns_client(self):
        """
        Connect to the SNS service
        """
        if not self.sns_client:
            boto3.client(
                service_name='sns',
                aws_access_key_id=settings.SNS.AWS_ACCESS_KEY_ID,
                aws_secret_access_key=settings.SNS.AWS_SECRET_ACCESS_KEY,
                region_name=settings.SNS.AWS_SMS_REGION,
                use_ssl=True
            )
        return self.sns_client

    def publish_message(self, message):
        """
        Method uses boto3 client via witch SMS message is send
        :param message: SMS message
        """
        sns_client = self._get_sns_client()
        publish_kwargs = {
            'PhoneNumber': force_text(message.recipient),
            'Message': message.content,
        }
        if settings.SNS.SENDER_ID:
            publish_kwargs.update({
                'MessageAttributes': {
                    'AWS.SNS.SMS.SenderID': {
                        'DataType': 'String',
                        'StringValue': settings.SNS.SENDER_ID,
                    }
                }
            })
        try:
            sns_client.publish(**publish_kwargs)
            self.update_message(message, state=OutputSMSMessage.STATE.SENT, sent_at=timezone.now())
        except Exception as ex:
            self.update_message(message, state=OutputSMSMessage.STATE.ERROR_NOT_SENT, error=force_text(ex))
            # Do not re-raise caught exception. We do not know exact exception to catch so we catch them all
            # and log them into database. Re-raise exception causes transaction rollback (lost of information about
            # exception).



import datetime
import logging

from django.core.management.base import BaseCommand
from django.db.models import F, Q
from django.utils.timezone import now

from chamber.utils.transaction import atomic_with_signals

from pymess.config import get_email_sender, settings
from pymess.models import EmailMessage


logger = logging.getLogger(__name__)


class Command(BaseCommand):
    """
    Command for pulling info of sent e-mails.
    """

    @atomic_with_signals
    def handle(self, *args, **options):
        email_sender = get_email_sender()

        delay = datetime.timedelta(seconds=settings.EMAIL_PULL_INFO_DELAY_SECONDS)
        messages = EmailMessage.objects.filter(
            # start_time = last_webhook_received_at + delay
            # info_changed_at < start_time
            # info_changed_at < last_webhook_received_at + delay
            Q(info_changed_at__isnull=True) | Q(info_changed_at__lt=F('last_webhook_received_at') + delay),
            last_webhook_received_at__lt=now() - delay,
            sent_at__gt=now() - datetime.timedelta(seconds=settings.EMAIL_PULL_INFO_MAX_TIMEOUT_FROM_SENT_SECONDS)
        ).order_by('-sent_at')
        self.stdout.write('Total number of emails to pull info: {}'.format(messages.count()))

        messages = messages[:settings.EMAIL_PULL_INFO_BATCH_SIZE]
        self.stdout.write('Number of emails in this batch: {}'.format(messages.count()))

        for message in messages:
            try:
                email_sender.pull_message_info(message)
            except Exception as ex:
                # Rollback should not be applied for already updated e-mails
                logger.exception(ex)
        self.stdout.write('DONE')



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2019-01-11 07:20
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pymess', '0003_dialer'),
    ]

    operations = [
        migrations.AddField(
            model_name='dialermessage',
            name='is_final_state',
            field=models.BooleanField(default=False, verbose_name='is done'),
        ),
    ]
